@@ -30,15 +30,18 @@ from __future__ import absolute_import
 import itertools
 import six
 import random
-from functools import wraps, partial
+from functools import partial
 
 import txaio
 
 from autobahn.util import ObservableMixin
-from autobahn.websocket.util import parse_url
+from autobahn.websocket.util import parse_url as parse_ws_url
+from autobahn.rawsocket.util import parse_url as parse_rs_url
+
 from autobahn.wamp.types import ComponentConfig, SubscribeOptions, RegisterOptions
 from autobahn.wamp.exception import SessionNotReady, ApplicationError
 from autobahn.wamp.auth import create_authenticator, IAuthenticator
+from autobahn.wamp.serializer import SERID_TO_SER
 
 
 __all__ = (
@@ -151,7 +154,7 @@ def _create_transport(index, transport, check_native_endpoint=None):
         # endpoint not required; we will deduce from URL if it's not provided
         # XXX not in the branch I rebased; can this go away? (is it redundant??)
         if 'endpoint' not in transport:
-            is_secure, host, port, resource, path, params = parse_url(transport['url'])
+            is_secure, host, port, resource, path, params = parse_ws_url(transport['url'])
             endpoint_config = {
                 'type': 'tcp',
                 'host': host,
@@ -173,7 +176,7 @@ def _create_transport(index, transport, check_native_endpoint=None):
                     isinstance(s, (six.text_type, str))
                     for s in transport['serializers']]):
                 raise ValueError("'serializers' must be a list of strings")
-            valid_serializers = ('msgpack', 'json')
+            valid_serializers = SERID_TO_SER.keys()
             for serial in transport['serializers']:
                 if serial not in valid_serializers:
                     raise ValueError(
@@ -182,12 +185,32 @@ def _create_transport(index, transport, check_native_endpoint=None):
                             ', '.join([repr(s) for s in valid_serializers]),
                         )
                     )
-        serializer_config = transport.get('serializers', [u'msgpack', u'json'])
+        serializer_config = transport.get('serializers', [u'cbor', u'json'])
 
     elif kind == 'rawsocket':
         if 'endpoint' not in transport:
-            raise ValueError("Missing 'endpoint' in transport")
-        endpoint_config = transport['endpoint']
+            if transport['url'].startswith('rs'):
+                # # try to parse RawSocket URL ..
+                isSecure, host, port = parse_rs_url(transport['url'])
+            elif transport['url'].startswith('ws'):
+                # try to parse WebSocket URL ..
+                isSecure, host, port, resource, path, params = parse_ws_url(transport['url'])
+            else:
+                raise RuntimeError()
+            if host == 'unix':
+                # here, "port" is actually holding the path on the host, eg "/tmp/file.sock"
+                endpoint_config = {
+                    'type': 'unix',
+                    'path': port,
+                }
+            else:
+                endpoint_config = {
+                    'type': 'tcp',
+                    'host': host,
+                    'port': port,
+                }
+        else:
+            endpoint_config = transport['endpoint']
         if 'serializers' in transport:
             raise ValueError("'serializers' is only for websocket; use 'serializer'")
         # always a list; len == 1 for rawsocket
@@ -196,7 +219,7 @@ def _create_transport(index, transport, check_native_endpoint=None):
                 raise ValueError("'serializer' must be a string")
             serializer_config = [transport['serializer']]
         else:
-            serializer_config = [u'msgpack']
+            serializer_config = [u'cbor']
 
     else:
         assert False, 'should not arrive here'
@@ -469,6 +492,11 @@ class Component(ObservableMixin):
         self._realm = realm
         self._extra = extra
 
+        self._delay_f = None
+        self._done_f = None
+        self._session = None
+        self._stopping = False
+
     def _can_reconnect(self):
         # check if any of our transport has any reconnect attempt left
         for transport in self._transports:
@@ -491,9 +519,21 @@ class Component(ObservableMixin):
             "done" or with an error if something went wrong.
         """
 
+        # we can only be "start()ed" once before we stop .. but that
+        # doesn't have to be an error we can give back another future
+        # that fires when our "real" _done_f is completed.
+        if self._done_f is not None:
+            d = txaio.create_future()
+
+            def _cb(arg):
+                txaio.resolve(d, arg)
+
+            txaio.add_callbacks(self._done_f, _cb, _cb)
+            return d
+
         # this future will be returned, and thus has the semantics
         # specified in the docstring.
-        done_f = txaio.create_future()
+        self._done_f = txaio.create_future()
 
         # Create a generator of transports that .can_reconnect()
         transport_gen = itertools.cycle(self._transports)
@@ -503,11 +543,20 @@ class Component(ObservableMixin):
         transport_candidate = [0]
 
         def error(fail):
-            self.log.info("Internal error {msg}", msg=txaio.failure_message(fail))
-            self.log.debug("{tb}", tb=txaio.failure_format_traceback(fail))
-            txaio.reject(done_f, fail)
+            self._delay_f = None
+            if self._stopping:
+                # might be better to add framework-specific checks in
+                # subclasses to see if this is CancelledError (for
+                # Twisted) and whatever asyncio does .. but tracking
+                # if we're in the shutdown path is fine too
+                txaio.resolve(self._done_f, None)
+            else:
+                self.log.info("Internal error {msg}", msg=txaio.failure_message(fail))
+                self.log.debug("{tb}", tb=txaio.failure_format_traceback(fail))
+                txaio.reject(self._done_f, fail)
 
         def attempt_connect(_):
+            self._delay_f = None
 
             def handle_connect_error(fail):
                 # FIXME - make txaio friendly
@@ -536,7 +585,10 @@ class Component(ObservableMixin):
                     # and reason are all strings, describing where
                     # and what the problem is. See err(3) for more
                     # information."
-                    self.log.error(u"TLS failure: {reason}", reason=fail.value.args[1])
+                    # (and 'args' is a 1-tuple containing the above
+                    # 3-tuple...)
+                    ssl_lib, ssl_func, ssl_reason = fail.value.args[0][0]
+                    self.log.error(u"TLS failure: {reason}", reason=ssl_reason)
                 else:
                     self.log.error(
                         u'Connection failed: {error}',
@@ -573,7 +625,7 @@ class Component(ObservableMixin):
                 txaio.add_callbacks(notify_f, None, handle_connect_error)
 
             def session_done(x):
-                txaio.resolve(done_f, None)
+                txaio.resolve(self._done_f, None)
 
             connect_f = self._connect_once(loop, transport_candidate[0])
             txaio.add_callbacks(connect_f, session_done, connect_error)
@@ -587,7 +639,7 @@ class Component(ObservableMixin):
                 try:
                     raise RuntimeError(err_msg)
                 except RuntimeError as e:
-                    txaio.reject(done_f, e)
+                    txaio.reject(self._done_f, e)
                     return
 
             while True:
@@ -605,16 +657,29 @@ class Component(ObservableMixin):
                 transport_delay=delay,
             )
 
-            delay_f = txaio.sleep(delay)
-            txaio.add_callbacks(delay_f, attempt_connect, error)
+            self._delay_f = txaio.sleep(delay)
+            txaio.add_callbacks(self._delay_f, attempt_connect, error)
 
         # issue our first event, then start the reconnect loop
         start_f = self.fire('start', loop, self)
         txaio.add_callbacks(start_f, transport_check, error)
-        return done_f
+        return self._done_f
 
     def stop(self):
-        return self._session.leave()
+        self._stopping = True
+        if self._session and self._session.is_attached():
+            self._session.leave()
+        elif self._delay_f:
+            # This cancel request will actually call the "error" callback of
+            # the _delay_f future. Nothing to worry about.
+            return txaio.as_future(txaio.cancel, self._delay_f)
+        # if (for some reason -- should we log warning here to figure
+        # out if this can evern happen?) we've not fired _done_f, we
+        # do that now (causing our "main" to exit, and thus react() to
+        # quit)
+        if not txaio.is_called(self._done_f):
+            txaio.resolve(self._done_f, None)
+        return txaio.create_future_success(None)
 
     def _connect_once(self, reactor, transport):
 
@@ -723,32 +788,21 @@ class Component(ObservableMixin):
 
         transport.connect_attempts += 1
 
-        d = self._connect_transport(reactor, transport, create_session)
+        d = self._connect_transport(reactor, transport, create_session, done)
 
-        def on_connect_sucess(proto):
-            # if e.g. an SSL handshake fails, we will have
-            # successfully connected (i.e. get here) but need to
-            # 'listen' for the "connectionLost" from the underlying
-            # protocol in case of handshake failure .. so we wrap
-            # it. Also, we don't increment transport.success_count
-            # here on purpose (because we might not succeed).
-            orig = proto.connectionLost
-
-            @wraps(orig)
-            def lost(fail):
-                rtn = orig(fail)
-                if not txaio.is_called(done):
-                    txaio.reject(done, fail)
-                return rtn
-            proto.connectionLost = lost
-
-        def on_connect_failure(err):
+        def on_error(err):
+            """
+            this may seem redundant after looking at _connect_transport, but
+            it will handle a case where something goes wrong in
+            _connect_transport itself -- as the only connect our
+            caller has is the 'done' future
+            """
             transport.connect_failures += 1
-            # failed to establish a connection in the first place
-            txaio.reject(done, err)
-
-        txaio.add_callbacks(d, on_connect_sucess, None)
-        txaio.add_callbacks(d, None, on_connect_failure)
+            # something bad has happened, and maybe didn't get caught
+            # upstream yet
+            if not txaio.is_called(done):
+                txaio.reject(done, err)
+        txaio.add_callbacks(d, None, on_error)
 
         return done
 
